var Protocol = function()
{
	var protocol = {};
	var debug = 0;
	
	var lysatesIsValid = false;

	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	
			Doc.show('all-protocol');
			
			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLatestReagentLotNumbers';		
			Wizard.showLoadingAnimation('Loading recent reagent lot numbers...');
			Wizard.asyncJsonRequest(url, protocol.reagentsLoaded);
			
			Events.addEventHandler('showReagents', 'click', protocol.toggleReagents);
		}
		else
		{
			Events.addEventHandler('lysates', 'change', protocol.lysatesOnChange);
			Buttons.addClickHandler('gocreate', protocol.viewProtocol);

			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetUnprocessedLysates';
			Wizard.showLoadingAnimation('Loading Lysate extracts...');
			Wizard.asyncJsonRequest(url, protocol.initializeStep1);
		}
	}


	protocol.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	
		var lysates = response.lysates;
		
		Doc.element('reserve').value = response.ReservedBy.id;
		Doc.element('YellowLabel').value = response.YellowLabel.id;
		
		var lysateList = frm.lysates;
		if (lysates.length > 0)
		{
			// Check if we have any lysate reserved by the current user
			var currentUser = Data.get(lysateList, 'current-user');
			var hasReserved = false;
			var numYellow = 0;
			for (var i=0; i < lysates.length; i++)
			{
				var lysate = lysates[i];
				if (lysate.ReservedBy == currentUser) hasReserved = true;
				if (lysate.specimen.YellowLabel != null) numYellow++;
			}
			
			var numSelected = 0;
			var maxNonYellow = 12-numYellow;
			for (var i=0; i < lysates.length; i++)
			{
				var lysate = lysates[i];
				var isYellow = lysate.specimen.YellowLabel != null;
				var name = (i+1) + ': ' + Strings.encodeTags(lysate.name);
				if (lysate.bioWell)
				{
					name += ' -- ' + Strings.encodeTags(lysate.bioWell.bioPlate.name + ' (' + lysate.bioWell.location+')');
				}
				if (lysate.ReservedBy)
				{
					name += ' ['+lysate.ReservedBy+']';
				}
				
				// We only auto-select items reserved by the current user (if it has any)
				// and never any items reserved by other users
				var selected = false;
				if (numSelected < 12)
				{
					if (hasReserved)
					{
						selected = lysate.ReservedBy == currentUser;
					}
					else
					{
						selected = isYellow || maxNonYellow > 0;
					}
					if (selected) 
					{
						numSelected++;
						if (!isYellow) maxNonYellow--;
					}
				}
				
				var option = new Option(name, lysate.id, selected, selected);
				if (isYellow) option.className = 'yellow';
				option.lysate = lysate;
				lysateList.options[lysateList.length] = option;
			}
			protocol.lysatesOnChange();
		}
		else
		{	
			Wizard.setFatalError('No Lysate available for processing.');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gocreate');
	}
	

	// Add pools to the pools list based on the bioplate selection
	protocol.lysatesOnChange = function()
	{
		lysatesIsValid = false;
		
		var frm = document.forms['reggie'];
		var lysateList = frm.lysates;
		
		var numSelected = 0;
		var numUnselectedYellow = 0;
		for (var i = 0; i < lysateList.length; i++)
		{
			if (lysateList[i].selected) 
			{
				numSelected++;
			}
			else if (lysateList[i].lysate.specimen.YellowLabel != null)
			{
				numUnselectedYellow++;
			}
		}
		
		if (numSelected > 12)
		{
			Wizard.setInputStatus('lysate', 'invalid', 'Must not select more than 12 items.');
			return;
		}
		else if (numSelected < 1)
		{
			Wizard.setInputStatus('lysate', 'invalid', 'Must select at least 1 item.');
			return;
		}
		
		lysatesIsValid = true;
		
		if (numUnselectedYellow > 0)
		{
			Wizard.setInputStatus('lysate', 'warning', 'Not all YellowLabel specimen are selected.');
		}
		else
		{
			Wizard.setInputStatus('lysate', 'valid');
		}
	}
	
	protocol.viewProtocol = function()
	{
		var frm = document.forms['reggie'];
		if (!lysatesIsValid)
		{
			Wizard.notifyError();
			return;
		}
		frm.submit();
	}

	protocol.reagentsLoaded = function(response)
	{
		var rna = response.rna;
		var lysate = response.lysate;
		if (rna) 
		{
			protocol.fillLatestReagents(rna);
			Doc.element('latestQiaCubeDate').innerHTML = Reggie.reformatDate(rna.QiaCubeDate);
			Doc.show('showLatestReagents', 'inline');
		}
		if (lysate) protocol.fillLatestReagents(lysate);
	}
	
	protocol.fillLatestReagents = function(item)
	{
		for (var rg in item)
		{
			if (item[rg] != null)
			{
				var e = Doc.element(rg);
				if (e)
				{
					e.innerHTML = Strings.encodeTags(item[rg]);
				}
				
			}
		}
	}
	
	protocol.toggleReagents = function(event)
	{
		Doc.addOrRemoveClass(document.body, 'hide-reagents', !event.currentTarget.checked);
	}

	return protocol;
}();

Doc.onLoad(Protocol.initPage);

